// Improved Delphi Hardware-Serialnumber-Generator
// by ViaThinkSoft [www.viathinksoft.com]
// and Daniel Marschall [www.daniel-marschall.de]

// Version 2.1
// Revision: 2009-08-09

// URL to the current release:
// http://www.viathinksoft.com/?page=projektanzeige&seite=download&id=149

// Source of original package:
// http://www.techblog.tomksoft.com/programming/hardware-serial-number-generator.html

// Improved and extended by Daniel Marschall [www.daniel-marschall.de]
// - Removed Main-Form Interaction
// - Added MAC-Adress support (now only 1 address)
// - Disk-Serial and Disk-Model: Only for hard disks, not for USB cardreaders
//   or other devices; the hardware id changes often and the system may hang if
//   no card is inserted.
// - Moved TRegistry.Create out of the Try-Finally-Block
// - Also the decoded hardware id can be returned, very good formated
// - Internally save the hwid value for performance
// - Third MD5 string can now check the integrity of the first two MD5s
// - The first two MD5 parts are now dependent of each other
// - Included a new Demo.exe to this unit package
// - Fully Delphi 2009 Compatible (MD5 unit is now AnsiString-Only)
// - Added memory and registry cache control
// - Added as VCL and packed everything in nice classes

unit HWSNGen;

// Original code and idea
// Hardware serial number generator unit, MIT license
// Kind wishes,
// Tomasz Klekot http://tomksoft.iblogger.org

{$INCLUDE DelphiVersion.inc}

interface

uses
  SysUtils, Windows, Classes, Registry2;

procedure Register;

type
  THardwareTokens = record // Kann man das auch irgendwie private machen?
    MainBoard: string;
    WindowsProductKey: string;
    WindowsInstallDate: string;
    Processor: string;
    BIOSID: string;
    HardDiskSerials: TStringList;
    HardDiskModels: TStringList;
    FirstMACAddress: string;
  end;

  // Auch als Persistent probiert... War aber mit der ganzen Referenzsemantik zu komplex
  THardwareIdentification = record
    HwCollection: string;
    HwHash: string;
  end;

  THardwareIdCache = {abstract} class(TPersistent)
  private
    FEnabled: boolean;
    function GetId: THardwareIdentification;
    procedure SetId(Value: THardwareIdentification);
  protected
    function GetIdentification: THardwareIdentification; virtual; abstract;
    procedure SetIdentification(Value: THardwareIdentification); virtual; abstract;
    procedure DeleteIdentification; virtual; abstract;
    procedure SetEnabled(Value: boolean);
    function GetEnabled: boolean;
  public
    procedure Clear;
    function IsEmpty: boolean;
    property HardwareIdentification: THardwareIdentification
      read GetId write SetId;
  published
    property Enabled: boolean
      read GetEnabled write SetEnabled default true;
  end;

  THardwareIdMemoryCache = class(THardwareIdCache)
  private
    FCache: THardwareIdentification;
  protected
    function GetIdentification: THardwareIdentification; override;
    procedure SetIdentification(Value: THardwareIdentification); override;
    procedure DeleteIdentification; override;
  public
    constructor Create;
  end;

  THardwareIdRegistryCache = class(THardwareIdCache)
  private
    FReg: TRegistry2;
    FIntegrityPassphrase: string;
    FUseCustomRegistryCache: boolean;
    FCustomRegistryCacheRootKey: THKey;
    FCustomRegistryCacheKey: string;
  protected
    function GetIdentification: THardwareIdentification; override;
    procedure SetIdentification(Value: THardwareIdentification); override;
    procedure DeleteIdentification; override;
    function GetEnabled: boolean;
    procedure SetEnabled(Value: boolean);
  public
    function RegistryKey: string;
    function RegistryRootKey: THKey;
    constructor Create;
    destructor Destroy; override;
  published
    property Enabled: boolean
      read GetEnabled write SetEnabled default false;
    property IntegrityPassphrase: string
      read FIntegrityPassphrase write FIntegrityPassphrase;
    property UseCustomRegistryCache: boolean
      read FUseCustomRegistryCache write FUseCustomRegistryCache
      default false;
    property CustomRegistryCacheRootKey: THKey
      read FCustomRegistryCacheRootKey write FCustomRegistryCacheRootKey
      default hkLocalMachine;
    property CustomRegistryCacheKey: string
      read FCustomRegistryCacheKey write FCustomRegistryCacheKey;
  end;

  TCustomHWSNGen = class(TComponent)
  private
    FMemoryCache: THardwareIdMemoryCache;
    FRegistryCache: THardwareIdRegistryCache;
    function ValidUpperCaseHexString(str: string): boolean;
    class function GetVersion: string;
  protected
    function AnalyzeComputerHardware: THardwareTokens;
    function CreateIntegrityHash(stringToVerify: AnsiString): string;
    function EncodeHwId(HardwareData: THardwareTokens): string;
    function HwIdToString(HardwareData: THardwareTokens): string;
    function GenerateHwId(): THardwareIdentification;
    property MemoryCache: THardwareIdMemoryCache
      read FMemoryCache write FMemoryCache;
    property RegistryCache: THardwareIdRegistryCache
      read FRegistryCache write FRegistryCache;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetHardwareId: THardwareIdentification;
    function IsValidHWID(HWID: string): boolean;
    procedure ClearCache;
    function CalculateNecessary: boolean;
    property GeneratorVersion: string
      read GetVersion {write NullDummy};
  end;

  THWSNGen = class(TCustomHWSNGen)
  published
    property MemoryCache;
    property RegistryCache;
  end;

implementation

uses
  MD5, magwmi, MacAddress, Registry;

type
  EIntegrityPassphraseException = class(Exception);

const
  STR_UNKNOWN = '(Unknown)';

function MyListToString(StringList: TStringList): string;
var
  i: integer;
begin
  if StringList.Count = 0 then
  begin
    result := STR_UNKNOWN;
  end
  else
  begin
    result := '';
    for i := 0 to StringList.Count - 1 do
    begin
      result := result + StringList.Strings[i];
      if i+1 < StringList.Count then result := result + #13#10;
    end;
  end;
end;

{ THardwareIdCache }

function THardwareIdCache.IsEmpty: boolean;
begin
  result := (HardwareIdentification.HwCollection = '') or
            (HardwareIdentification.HwHash = '');
end;

procedure THardwareIdCache.Clear;
begin
  DeleteIdentification;
end;

function THardwareIdCache.GetId: THardwareIdentification;
begin
  if FEnabled then
  begin
    result := GetIdentification;
  end;
end;

procedure THardwareIdCache.SetId(Value: THardwareIdentification);
begin
  if FEnabled then
  begin
    SetIdentification(Value);
  end;
end;

procedure THardwareIdCache.SetEnabled(Value: boolean);
begin
  FEnabled := Value;
end;

function THardwareIdCache.GetEnabled: boolean;
begin
  result := FEnabled;
end;

{ THardwareIdMemoryCache }

constructor THardwareIdMemoryCache.Create;
begin
  inherited;

  FCache.HwCollection := '';
  FCache.HwHash := '';
end;

function THardwareIdMemoryCache.GetIdentification: THardwareIdentification;
begin
  result := FCache;
end;

procedure THardwareIdMemoryCache.SetIdentification(Value: THardwareIdentification);
begin
  FCache := Value;
end;

procedure THardwareIdMemoryCache.DeleteIdentification;
begin
  FCache.HwCollection := '';
  FCache.HwHash := '';
end;

{ THardwareIdRegistryCache }

const
  REG_COLLECTION = 'Collection';
  REG_HASH = 'Hash';
  REG_TIME = 'Timestamp';
  // To prevent manipulation of the registry cache
  REG_INTEGRITY = 'Integrity';
  // If changed, the integrity string is not valid anymore
  REG_INTEGRITY_SALT = '1{[Rasgmlk.##KSFDA';

function THardwareIdRegistryCache.RegistryKey: string;
const
  // V2 Keys are self-integrity
  REG_KEY = '\Software\ViaThinkSoft\HWSNGen\%s\Cache\%s\';
  // If changed, the registry key are different with the same passphrase
  PHS_SALT = '")2u$wifa"$&%/(Q!j ';
begin
  if UseCustomRegistryCache then
  begin
    result := CustomRegistryCacheKey;
  end
  else
  begin
    result := Format(REG_KEY, [THwSnGen.GetVersion, UpperAnsiMD5(PHS_SALT+IntegrityPassphrase)]);
  end;
end;

function THardwareIdRegistryCache.RegistryRootKey: THKey;
begin
  if UseCustomRegistryCache then
  begin
    result := CustomRegistryCacheRootKey;
  end
  else
  begin
    result := hkLocalMachine;
  end;
end;

constructor THardwareIdRegistryCache.Create;
begin
  inherited;

  FReg := TRegistry2.Create;
  CustomRegistryCacheRootKey := hkLocalMachine;
end;

destructor THardwareIdRegistryCache.Destroy;
begin
  FReg.Free;

  inherited;
end;

function THardwareIdRegistryCache.GetIdentification: THardwareIdentification;
var
  hash: string;
  col: string;
begin
  FReg.RootKey := RegistryRootKey;
  if FReg.OpenKeyReadOnly(RegistryKey) then
  begin
    col := FReg.ReadString(REG_COLLECTION);
    hash := FReg.ReadString(REG_HASH);
    if FReg.ReadString(REG_INTEGRITY) = UpperAnsiMD5(REG_INTEGRITY_SALT +
      IntegrityPassphrase + hash + col) then
    begin
      result.HwCollection := col;
      result.HwHash := hash;
    end;
    FReg.CloseKey;
  end;
end;

procedure THardwareIdRegistryCache.SetIdentification(Value: THardwareIdentification);
begin
  FReg.RootKey := RegistryRootKey;
  if FReg.OpenKey(RegistryKey, true) then
  begin
    FReg.WriteString(REG_COLLECTION, Value.HwCollection);
    FReg.WriteString(REG_HASH, Value.HwHash);
    FReg.WriteString(REG_INTEGRITY, UpperAnsiMD5(REG_INTEGRITY_SALT +
      IntegrityPassphrase + Value.HwHash + Value.HwCollection));
    FReg.WriteDateTime(REG_TIME, Now());
    FReg.CloseKey;
  end;
end;

procedure THardwareIdRegistryCache.DeleteIdentification;
var
  Info: TRegKeyInfo;
  maydelete: boolean;
begin
  FReg.RootKey := RegistryRootKey;

  // Damit wrden Unterschlssel mitgelscht werden (bei shared Keys)
  // FReg.DeleteKey(RegistryKey);

  maydelete := false;
  if FReg.OpenKey(RegistryKey, true) then
  begin
    FReg.DeleteValue(REG_COLLECTION);
    FReg.DeleteValue(REG_HASH);
    FReg.DeleteValue(REG_INTEGRITY);
    FReg.DeleteValue(REG_TIME);
    maydelete := FReg.GetKeyInfo(Info) and
      (Info.NumValues = 0) and (Info.NumSubKeys = 0);
    FReg.CloseKey;
  end;

  if maydelete then FReg.DeleteKey(RegistryKey);
end;

resourcestring
  LNG_INTEGRITY_PASSPHRASE_EXCEPTION = 'The registry cache cannot be enabled ' +
    'if you have not set an integrity passphrase. This is neccessary because ' +
    'then the registry content cannot be trusted since they can be easily ' +
    'manipulated.';

procedure THardwareIdRegistryCache.SetEnabled(Value: boolean);
begin
  if Value and (IntegrityPassphrase = '') then
  begin
    raise EIntegrityPassphraseException.Create(LNG_INTEGRITY_PASSPHRASE_EXCEPTION);
  end
  else
  begin
    inherited Enabled := Value;
  end;
end;

function THardwareIdRegistryCache.GetEnabled: boolean;
begin
  result := inherited Enabled;
end;

{ TCustomHWSNGen }

// Note: Change the integrity salt to make Strings from older versions invalid.
// Usually the hash salt's don't have to be changed.

const
  INSIDE_INTEGRITY_SALT = ')$"(KDASF2904uJHfF';
  HASH_SALT = 'QUHsd#ddfddf3%75fd;]';
  VER_PREFIX = 'VHID21'; // Always 6 byte long
  SEP = '-';

constructor TCustomHWSNGen.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  
  MemoryCache := THardwareIdMemoryCache.Create;
  MemoryCache.Enabled := true;
  RegistryCache := THardwareIdRegistryCache.Create;
  RegistryCache.Enabled := false;
end;

destructor TCustomHWSNGen.Destroy;
begin
  RegistryCache.Free;
  MemoryCache.Free;

  inherited Destroy;
end;

class function TCustomHWSNGen.GetVersion: string;
begin
  result := '2.1';
end;

function TCustomHWSNGen.AnalyzeComputerHardware(): THardwareTokens;
var
  s: string;
  sa: TStringList;
  i: Integer;
  // macs: TStringList;
  proc_desc, proc_id: string;
const
  PROC_STR = '%s - ID: %s';
begin
  s:='';

  result.MainBoard := MagWmiGetBaseBoard;
  if (result.MainBoard = '') or (result.MainBoard = 'NULL') then
    result.MainBoard := MagWmiGetBaseBoard;
  if (result.MainBoard = '') or (result.MainBoard = 'NULL') then
    result.MainBoard := STR_UNKNOWN;

  result.WindowsProductKey := MagWmiGetOSInfo('SerialNumber');
  if (result.WindowsProductKey = '') or (result.WindowsProductKey = 'NULL') then
    result.WindowsProductKey := MagWmiGetOSInfo('SerialNumber');
  if (result.WindowsProductKey = '') or (result.WindowsProductKey = 'NULL') then
    result.WindowsProductKey := STR_UNKNOWN;

  result.WindowsInstallDate := MagWmiGetOSInfo('InstallDate');
  if (result.WindowsInstallDate = '') or (result.WindowsInstallDate = 'NULL') then
    result.WindowsInstallDate := MagWmiGetOSInfo('InstallDate');
  if (result.WindowsInstallDate = '') or (result.WindowsInstallDate = 'NULL') then
    result.WindowsInstallDate := STR_UNKNOWN;

  proc_desc := MagWmiGetProcInfo('Description');
  if (proc_desc = '') or (proc_desc = 'NULL') then
    proc_desc := MagWmiGetProcInfo('Description');
  if (proc_desc = '') or (proc_desc = 'NULL') then
    proc_desc := STR_UNKNOWN;

  proc_id := MagWmiGetProcInfo('ProcessorID');
  if (proc_id = '') or (proc_id = 'NULL') then
    proc_id := MagWmiGetProcInfo('ProcessorID');
  if (proc_id = '') or (proc_id = 'NULL') then
    proc_id := STR_UNKNOWN;

  result.Processor := Format(PROC_STR, [proc_desc, proc_id]);

  result.BIOSID := MagWmiGetBIOSVersion;
  if (result.BIOSID = '') or (result.BIOSID = 'NULL') then
    result.BIOSID := MagWmiGetBIOSVersion;
  if (result.BIOSID = '') or (result.BIOSID = 'NULL') then
    result.BIOSID := STR_UNKNOWN;

  result.HardDiskSerials := TStringList.Create;
  sa := MagWmiGetHardDiskSerials();
  for i := 0 to sa.Count - 1 do
  begin
    if (sa[i] <> '') or (sa[i] <> 'NULL') then
    begin
      result.HardDiskSerials.Add(sa[i]);
    end;
  end;

  result.HardDiskModels := TStringList.Create;
  sa := MagWmiGetHardDiskModels();
  for i := 0 to sa.Count - 1 do
  begin
    if (sa[i] <> '') or (sa[i] <> 'NULL') then
    begin
      result.HardDiskModels.Add(sa[i]);
    end;
  end;

  { result.MACAddresses := TStringList.Create;
  macs := Get_EthernetAddresses();
  for i := 0 to macs.count - 1 do
  begin
    if (sa[i] <> '') or (sa[i] <> 'NULL') then
    begin
      result.MACAddresses.Add(sa[i]);
    end;
  end; }

  // Read out only the first address. Because users can add or remove
  // mac-addresses when all mac's are included
  result.FirstMACAddress := GetMACAddress();
  if (result.FirstMACAddress = '') then
    result.FirstMACAddress := STR_UNKNOWN;
end;

procedure TCustomHWSNGen.ClearCache;
begin
  RegistryCache.Clear;
  MemoryCache.Clear;
end;

function TCustomHWSNGen.CreateIntegrityHash(stringToVerify: AnsiString): string;
begin
  result := UpperAnsiMD5(INSIDE_INTEGRITY_SALT + stringToVerify);
  result := Copy(result, 1, 16); // MD5 "halbieren"
end;

function TCustomHWSNGen.EncodeHwId(HardwareData: THardwareTokens): string;
var
  identification, verify: string;
begin
  identification := UpperAnsiMD5(HASH_SALT +
    HardwareData.MainBoard +
    HardwareData.WindowsProductKey +
    HardwareData.WindowsInstallDate +
    HardwareData.Processor +
    HardwareData.BIOSID +
    HardwareData.HardDiskSerials.CommaText +
    HardwareData.HardDiskModels.CommaText +
    HardwareData.FirstMACAddress);

  // Integrity control
  verify := CreateIntegrityHash(identification);

  Result := VER_PREFIX + SEP + identification + SEP + verify;
end;

function TCustomHWSNGen.HwIdToString(HardwareData: THardwareTokens): string;
const
  DescStr = 'Motherboard:' + #13#10 + '%s' + #13#10#13#10 +
            'Windows Product Key:' + #13#10 + '%s' + #13#10#13#10 +
            'Windows Install Date:' + #13#10 + '%s' + #13#10#13#10 +
            'Processor ID:' + #13#10 + '%s' + #13#10#13#10 +
            'BIOS ID:' + #13#10 + '%s' + #13#10#13#10 +
            'Hard Disk Serials:' + #13#10 + '%s' + #13#10#13#10 +
            'Hard Disk Models:' + #13#10 + '%s' + #13#10#13#10 +
            'First MAC Address:' + #13#10 + '%s';
begin
  result := Format(DescStr, [HardwareData.MainBoard,
              HardwareData.WindowsProductKey,
              HardwareData.WindowsInstallDate,
              HardwareData.Processor,
              HardwareData.BIOSID,
              MyListToString(HardwareData.HardDiskSerials),
              MyListToString(HardwareData.HardDiskModels),
              HardwareData.FirstMACAddress]);
end;

function TCustomHWSNGen.GenerateHwId(): THardwareIdentification;
var
  HardwareTokens: THardwareTokens;
begin
  HardwareTokens := AnalyzeComputerHardware();

  result.HwCollection := HwIdToString(HardwareTokens);                       
  result.HwHash := EncodeHwId(HardwareTokens);
end;

function TCustomHWSNGen.ValidUpperCaseHexString(str: string): boolean;
var
  i: integer;
begin
  result := false;
  for i := 1 to Length(str) do
  begin
    {$IFDEF DELPHI_2009_UP}
    if not CharInSet(str[i], ['A'..'F', '0'..'9']) then exit;
    {$ELSE}
    if not (str[i] in ['A'..'F', '0'..'9']) then exit;
    {$ENDIF}
  end;
  result := true;
end;

function TCustomHWSNGen.GetHardwareId: THardwareIdentification;
begin
  if MemoryCache.Enabled and (not MemoryCache.IsEmpty) then
  begin
    result := MemoryCache.HardwareIdentification;
    exit;
  end;

  if RegistryCache.Enabled and (not RegistryCache.IsEmpty) then
  begin
    result := RegistryCache.HardwareIdentification;
    exit;
  end;

  result := GenerateHwId();

  if MemoryCache.Enabled then
  begin
    MemoryCache.HardwareIdentification := result;
  end;

  if RegistryCache.Enabled then
  begin
    RegistryCache.HardwareIdentification := result;
  end;
end;

function TCustomHWSNGen.IsValidHWID(HWID: string): boolean;
var
  identification, verify: string;
begin
  result := false;

  // Step 1: Length OK?

  if Length(HWID) <> length(VER_PREFIX)+1+32+1+16 then exit;

  // Step 2: Is the prefix OK?

  if Copy(HWID, 1, length(VER_PREFIX)) <> VER_PREFIX then exit;

  if Copy(HWID, 1+length(VER_PREFIX), 1) <> SEP then exit;

  // Extract MD5 parts

  identification := Copy(HWID, 1+length(VER_PREFIX)+1, 32);

  if Copy(HWID, 1+length(VER_PREFIX)+1+32, 1) <> SEP then exit;

  verify := Copy(HWID, 1+length(VER_PREFIX)+1+32+1, 16);

  // Step 3: Only valid characters?

  if not ValidUpperCaseHexString(identification) then exit;

  // Step 4: Integrity match?

  if verify <> CreateIntegrityHash(identification) then exit;

  // Step 5: Integrity MD5 OK? (Actually unneccessary because of Step 4)

  if not ValidUpperCaseHexString(verify) then exit;

  result := true;
end;

function TCustomHWSNGen.CalculateNecessary: boolean;
begin
  result := ((not MemoryCache.Enabled) or (MemoryCache.IsEmpty)) and
            ((not RegistryCache.Enabled) or (RegistryCache.IsEmpty));
end;

procedure Register;
begin
  RegisterComponents('ViaThinkSoft', [THWSNGen]);
end;

end.

